import sys
from datetime import datetime
from typing import Dict, List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

ApplicationArn = str
ApplicationId = str
ApplicationName = str
AutoStopConfigIdleTimeoutMinutesInteger = int
Boolean = bool
ClientToken = str
CpuSize = str
DiskSize = str
Double = float
EncryptionKeyArn = str
EngineType = str
EntryPointArgument = str
EntryPointPath = str
HiveCliParameters = str
IAMRoleArn = str
InitScriptPath = str
Integer = int
JobArn = str
JobRunId = str
JobRunType = str
ListApplicationsRequestMaxResultsInteger = int
ListJobRunsRequestMaxResultsInteger = int
MemorySize = str
NextToken = str
Query = str
ReleaseLabel = str
RequestIdentityUserArn = str
ResourceArn = str
SecurityGroupString = str
SparkSubmitParameters = str
String1024 = str
String256 = str
SubnetString = str
TagKey = str
TagValue = str
UriString = str
Url = str
WorkerTypeString = str


class ApplicationState(str):
    CREATING = "CREATING"
    CREATED = "CREATED"
    STARTING = "STARTING"
    STARTED = "STARTED"
    STOPPING = "STOPPING"
    STOPPED = "STOPPED"
    TERMINATED = "TERMINATED"


class Architecture(str):
    ARM64 = "ARM64"
    X86_64 = "X86_64"


class JobRunState(str):
    SUBMITTED = "SUBMITTED"
    PENDING = "PENDING"
    SCHEDULED = "SCHEDULED"
    RUNNING = "RUNNING"
    SUCCESS = "SUCCESS"
    FAILED = "FAILED"
    CANCELLING = "CANCELLING"
    CANCELLED = "CANCELLED"


class ConflictException(ServiceException):
    code: str = "ConflictException"
    sender_fault: bool = True
    status_code: int = 409


class InternalServerException(ServiceException):
    code: str = "InternalServerException"
    sender_fault: bool = False
    status_code: int = 500


class ResourceNotFoundException(ServiceException):
    code: str = "ResourceNotFoundException"
    sender_fault: bool = True
    status_code: int = 404


class ServiceQuotaExceededException(ServiceException):
    code: str = "ServiceQuotaExceededException"
    sender_fault: bool = True
    status_code: int = 402


class ValidationException(ServiceException):
    code: str = "ValidationException"
    sender_fault: bool = True
    status_code: int = 400


SecurityGroupIds = List[SecurityGroupString]
SubnetIds = List[SubnetString]


class NetworkConfiguration(TypedDict, total=False):
    subnetIds: Optional[SubnetIds]
    securityGroupIds: Optional[SecurityGroupIds]


class AutoStopConfig(TypedDict, total=False):
    enabled: Optional[Boolean]
    idleTimeoutMinutes: Optional[AutoStopConfigIdleTimeoutMinutesInteger]


class AutoStartConfig(TypedDict, total=False):
    enabled: Optional[Boolean]


TagMap = Dict[TagKey, TagValue]
Date = datetime


class MaximumAllowedResources(TypedDict, total=False):
    cpu: CpuSize
    memory: MemorySize
    disk: Optional[DiskSize]


class WorkerResourceConfig(TypedDict, total=False):
    cpu: CpuSize
    memory: MemorySize
    disk: Optional[DiskSize]


WorkerCounts = int


class InitialCapacityConfig(TypedDict, total=False):
    workerCount: WorkerCounts
    workerConfiguration: Optional[WorkerResourceConfig]


InitialCapacityConfigMap = Dict[WorkerTypeString, InitialCapacityConfig]
Application = TypedDict(
    "Application",
    {
        "applicationId": ApplicationId,
        "name": Optional[ApplicationName],
        "arn": ApplicationArn,
        "releaseLabel": ReleaseLabel,
        "type": EngineType,
        "state": ApplicationState,
        "stateDetails": Optional[String256],
        "initialCapacity": Optional[InitialCapacityConfigMap],
        "maximumCapacity": Optional[MaximumAllowedResources],
        "createdAt": Date,
        "updatedAt": Date,
        "tags": Optional[TagMap],
        "autoStartConfiguration": Optional[AutoStartConfig],
        "autoStopConfiguration": Optional[AutoStopConfig],
        "networkConfiguration": Optional[NetworkConfiguration],
        "architecture": Optional[Architecture],
    },
    total=False,
)
ApplicationSummary = TypedDict(
    "ApplicationSummary",
    {
        "id": ApplicationId,
        "name": Optional[ApplicationName],
        "arn": ApplicationArn,
        "releaseLabel": ReleaseLabel,
        "type": EngineType,
        "state": ApplicationState,
        "stateDetails": Optional[String256],
        "createdAt": Date,
        "updatedAt": Date,
        "architecture": Optional[Architecture],
    },
    total=False,
)
ApplicationList = List[ApplicationSummary]
ApplicationStateSet = List[ApplicationState]


class CancelJobRunRequest(ServiceRequest):
    applicationId: ApplicationId
    jobRunId: JobRunId


class CancelJobRunResponse(TypedDict, total=False):
    applicationId: ApplicationId
    jobRunId: JobRunId


class Configuration(TypedDict, total=False):
    classification: "String1024"
    properties: Optional["SensitivePropertiesMap"]
    configurations: Optional["ConfigurationList"]


ConfigurationList = List[Configuration]
SensitivePropertiesMap = Dict[String1024, String1024]


class ManagedPersistenceMonitoringConfiguration(TypedDict, total=False):
    enabled: Optional[Boolean]
    encryptionKeyArn: Optional[EncryptionKeyArn]


class S3MonitoringConfiguration(TypedDict, total=False):
    logUri: Optional[UriString]
    encryptionKeyArn: Optional[EncryptionKeyArn]


class MonitoringConfiguration(TypedDict, total=False):
    s3MonitoringConfiguration: Optional[S3MonitoringConfiguration]
    managedPersistenceMonitoringConfiguration: Optional[ManagedPersistenceMonitoringConfiguration]


class ConfigurationOverrides(TypedDict, total=False):
    applicationConfiguration: Optional[ConfigurationList]
    monitoringConfiguration: Optional[MonitoringConfiguration]


CreateApplicationRequest = TypedDict(
    "CreateApplicationRequest",
    {
        "name": Optional[ApplicationName],
        "releaseLabel": ReleaseLabel,
        "type": EngineType,
        "clientToken": ClientToken,
        "initialCapacity": Optional[InitialCapacityConfigMap],
        "maximumCapacity": Optional[MaximumAllowedResources],
        "tags": Optional[TagMap],
        "autoStartConfiguration": Optional[AutoStartConfig],
        "autoStopConfiguration": Optional[AutoStopConfig],
        "networkConfiguration": Optional[NetworkConfiguration],
        "architecture": Optional[Architecture],
    },
    total=False,
)


class CreateApplicationResponse(TypedDict, total=False):
    applicationId: ApplicationId
    name: Optional[ApplicationName]
    arn: ApplicationArn


class DeleteApplicationRequest(ServiceRequest):
    applicationId: ApplicationId


class DeleteApplicationResponse(TypedDict, total=False):
    pass


Duration = int
EntryPointArguments = List[EntryPointArgument]


class GetApplicationRequest(ServiceRequest):
    applicationId: ApplicationId


class GetApplicationResponse(TypedDict, total=False):
    application: Application


class GetDashboardForJobRunRequest(ServiceRequest):
    applicationId: ApplicationId
    jobRunId: JobRunId


class GetDashboardForJobRunResponse(TypedDict, total=False):
    url: Optional[Url]


class GetJobRunRequest(ServiceRequest):
    applicationId: ApplicationId
    jobRunId: JobRunId


class TotalResourceUtilization(TypedDict, total=False):
    vCPUHour: Optional[Double]
    memoryGBHour: Optional[Double]
    storageGBHour: Optional[Double]


class Hive(TypedDict, total=False):
    query: Query
    initQueryFile: Optional[InitScriptPath]
    parameters: Optional[HiveCliParameters]


class SparkSubmit(TypedDict, total=False):
    entryPoint: EntryPointPath
    entryPointArguments: Optional[EntryPointArguments]
    sparkSubmitParameters: Optional[SparkSubmitParameters]


class JobDriver(TypedDict, total=False):
    sparkSubmit: Optional[SparkSubmit]
    hive: Optional[Hive]


class JobRun(TypedDict, total=False):
    applicationId: ApplicationId
    jobRunId: JobRunId
    name: Optional[String256]
    arn: JobArn
    createdBy: RequestIdentityUserArn
    createdAt: Date
    updatedAt: Date
    executionRole: IAMRoleArn
    state: JobRunState
    stateDetails: String256
    releaseLabel: ReleaseLabel
    configurationOverrides: Optional[ConfigurationOverrides]
    jobDriver: JobDriver
    tags: Optional[TagMap]
    totalResourceUtilization: Optional[TotalResourceUtilization]
    networkConfiguration: Optional[NetworkConfiguration]
    totalExecutionDurationSeconds: Optional[Integer]


class GetJobRunResponse(TypedDict, total=False):
    jobRun: JobRun


JobRunStateSet = List[JobRunState]
JobRunSummary = TypedDict(
    "JobRunSummary",
    {
        "applicationId": ApplicationId,
        "id": JobRunId,
        "name": Optional[String256],
        "arn": JobArn,
        "createdBy": RequestIdentityUserArn,
        "createdAt": Date,
        "updatedAt": Date,
        "executionRole": IAMRoleArn,
        "state": JobRunState,
        "stateDetails": String256,
        "releaseLabel": ReleaseLabel,
        "type": Optional[JobRunType],
    },
    total=False,
)
JobRuns = List[JobRunSummary]


class ListApplicationsRequest(ServiceRequest):
    nextToken: Optional[NextToken]
    maxResults: Optional[ListApplicationsRequestMaxResultsInteger]
    states: Optional[ApplicationStateSet]


class ListApplicationsResponse(TypedDict, total=False):
    applications: ApplicationList
    nextToken: Optional[NextToken]


class ListJobRunsRequest(ServiceRequest):
    applicationId: ApplicationId
    nextToken: Optional[NextToken]
    maxResults: Optional[ListJobRunsRequestMaxResultsInteger]
    createdAtAfter: Optional[Date]
    createdAtBefore: Optional[Date]
    states: Optional[JobRunStateSet]


class ListJobRunsResponse(TypedDict, total=False):
    jobRuns: JobRuns
    nextToken: Optional[NextToken]


class ListTagsForResourceRequest(ServiceRequest):
    resourceArn: ResourceArn


class ListTagsForResourceResponse(TypedDict, total=False):
    tags: Optional[TagMap]


class StartApplicationRequest(ServiceRequest):
    applicationId: ApplicationId


class StartApplicationResponse(TypedDict, total=False):
    pass


class StartJobRunRequest(ServiceRequest):
    applicationId: ApplicationId
    clientToken: ClientToken
    executionRoleArn: IAMRoleArn
    jobDriver: Optional[JobDriver]
    configurationOverrides: Optional[ConfigurationOverrides]
    tags: Optional[TagMap]
    executionTimeoutMinutes: Optional[Duration]
    name: Optional[String256]


class StartJobRunResponse(TypedDict, total=False):
    applicationId: ApplicationId
    jobRunId: JobRunId
    arn: JobArn


class StopApplicationRequest(ServiceRequest):
    applicationId: ApplicationId


class StopApplicationResponse(TypedDict, total=False):
    pass


TagKeyList = List[TagKey]


class TagResourceRequest(ServiceRequest):
    resourceArn: ResourceArn
    tags: TagMap


class TagResourceResponse(TypedDict, total=False):
    pass


class UntagResourceRequest(ServiceRequest):
    resourceArn: ResourceArn
    tagKeys: TagKeyList


class UntagResourceResponse(TypedDict, total=False):
    pass


class UpdateApplicationRequest(ServiceRequest):
    applicationId: ApplicationId
    clientToken: ClientToken
    initialCapacity: Optional[InitialCapacityConfigMap]
    maximumCapacity: Optional[MaximumAllowedResources]
    autoStartConfiguration: Optional[AutoStartConfig]
    autoStopConfiguration: Optional[AutoStopConfig]
    networkConfiguration: Optional[NetworkConfiguration]
    architecture: Optional[Architecture]


class UpdateApplicationResponse(TypedDict, total=False):
    application: Application


class EmrServerlessApi:

    service = "emr-serverless"
    version = "2021-07-13"

    @handler("CancelJobRun")
    def cancel_job_run(
        self, context: RequestContext, application_id: ApplicationId, job_run_id: JobRunId
    ) -> CancelJobRunResponse:
        raise NotImplementedError

    @handler("CreateApplication", expand=False)
    def create_application(
        self, context: RequestContext, request: CreateApplicationRequest
    ) -> CreateApplicationResponse:
        raise NotImplementedError

    @handler("DeleteApplication")
    def delete_application(
        self, context: RequestContext, application_id: ApplicationId
    ) -> DeleteApplicationResponse:
        raise NotImplementedError

    @handler("GetApplication")
    def get_application(
        self, context: RequestContext, application_id: ApplicationId
    ) -> GetApplicationResponse:
        raise NotImplementedError

    @handler("GetDashboardForJobRun")
    def get_dashboard_for_job_run(
        self, context: RequestContext, application_id: ApplicationId, job_run_id: JobRunId
    ) -> GetDashboardForJobRunResponse:
        raise NotImplementedError

    @handler("GetJobRun")
    def get_job_run(
        self, context: RequestContext, application_id: ApplicationId, job_run_id: JobRunId
    ) -> GetJobRunResponse:
        raise NotImplementedError

    @handler("ListApplications")
    def list_applications(
        self,
        context: RequestContext,
        next_token: NextToken = None,
        max_results: ListApplicationsRequestMaxResultsInteger = None,
        states: ApplicationStateSet = None,
    ) -> ListApplicationsResponse:
        raise NotImplementedError

    @handler("ListJobRuns")
    def list_job_runs(
        self,
        context: RequestContext,
        application_id: ApplicationId,
        next_token: NextToken = None,
        max_results: ListJobRunsRequestMaxResultsInteger = None,
        created_at_after: Date = None,
        created_at_before: Date = None,
        states: JobRunStateSet = None,
    ) -> ListJobRunsResponse:
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(
        self, context: RequestContext, resource_arn: ResourceArn
    ) -> ListTagsForResourceResponse:
        raise NotImplementedError

    @handler("StartApplication")
    def start_application(
        self, context: RequestContext, application_id: ApplicationId
    ) -> StartApplicationResponse:
        raise NotImplementedError

    @handler("StartJobRun")
    def start_job_run(
        self,
        context: RequestContext,
        application_id: ApplicationId,
        client_token: ClientToken,
        execution_role_arn: IAMRoleArn,
        job_driver: JobDriver = None,
        configuration_overrides: ConfigurationOverrides = None,
        tags: TagMap = None,
        execution_timeout_minutes: Duration = None,
        name: String256 = None,
    ) -> StartJobRunResponse:
        raise NotImplementedError

    @handler("StopApplication")
    def stop_application(
        self, context: RequestContext, application_id: ApplicationId
    ) -> StopApplicationResponse:
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(
        self, context: RequestContext, resource_arn: ResourceArn, tags: TagMap
    ) -> TagResourceResponse:
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_arn: ResourceArn, tag_keys: TagKeyList
    ) -> UntagResourceResponse:
        raise NotImplementedError

    @handler("UpdateApplication")
    def update_application(
        self,
        context: RequestContext,
        application_id: ApplicationId,
        client_token: ClientToken,
        initial_capacity: InitialCapacityConfigMap = None,
        maximum_capacity: MaximumAllowedResources = None,
        auto_start_configuration: AutoStartConfig = None,
        auto_stop_configuration: AutoStopConfig = None,
        network_configuration: NetworkConfiguration = None,
        architecture: Architecture = None,
    ) -> UpdateApplicationResponse:
        raise NotImplementedError
